@@ -5,6 +5,7 @@ from typing import TYPE_CHECKING, Iterable, List, Mapping, Optional, Union
 import pyarrow as pa
 
 from chalk.features.dataframe import DataFrame
+from chalk.utils.df_utils import pa_table_to_pl_df
 from chalk.utils.missing_dependency import missing_dependency_exception
 
 if TYPE_CHECKING:
@@ -35,12 +36,11 @@ def query_arrow_table(query: str, table: pa.Table) -> pa.Table:
     except ModuleNotFoundError:
         raise missing_dependency_exception("chalkpy[runtime]")
     table_names = list({t.name for t in sqlglot.parse_one(query, read="duckdb").find_all(sqlglot.expressions.Table)})
-    con = duckdb.connect(database=":memory:")
-    if len(table_names) > 1:
-        raise ValueError(f"Query should target only one table. Found: {', '.join(table_names)}")
-    if len(table_names) == 1:
+    if len(table_names) != 1:
+        raise ValueError(f"Query should target exactly one table. Found: {', '.join(table_names)}")
+    with duckdb.connect(database=":memory:") as con:
         con.register(table_names[0], table)
-    result_table = con.execute(query).fetch_arrow_table()
+        result_table = con.execute(query).fetch_arrow_table()
     return result_table
 
 
@@ -53,6 +53,7 @@ def query_as_features(
         import polars as pl
     except ImportError:
         raise missing_dependency_exception("chalkpy[runtime]")
+    del pl  # unused
     res = query_arrow_table(query, table)
     col_feature_mappings = _resolve_implicit_col_feature_mappings_str(
         # Note: snowflake by default capitalizes all table and column names, although it's
@@ -63,8 +64,7 @@ def query_as_features(
         expected_features=fqn_to_name.values(),
     )
     name_to_fqn = {v: k for k, v in fqn_to_name.items()}
-    df = pl.from_arrow(res)
-    assert isinstance(df, pl.DataFrame)
+    df = pa_table_to_pl_df(res)
     renamed = df.drop([col for col, f in zip(res.column_names, col_feature_mappings) if f is None]).rename(
         {col: name_to_fqn[f] for col, f in zip(res.column_names, col_feature_mappings) if f is not None}
     )
