@@ -40,6 +40,8 @@ from chalk.parsed.duplicate_input_gql import (
 
 
 def _convert_series(series: SeriesBase, chart_name: str) -> CreateMetricConfigSeriesGQL:
+    for validation_fn in series._validations:
+        validation_fn()
     return CreateMetricConfigSeriesGQL(
         metric=convert_metric_kind(series._metric, chart_name, series._name),
         filters=[_convert_filter(series_filter) for series_filter in series._filters],
