@@ -26,7 +26,10 @@ def _cached_getter(
     if len(cache) == 1:
         return cache[0]
     val = getter(instance)
-    cache.append(val)
+    if len(cache) > 0:
+        cache[0] = val
+    else:
+        cache.append(val)
     return val
 
 
@@ -38,7 +41,7 @@ def classproperty_support(cls: Type[T]) -> Type[T]:
     """
 
     # From https://stackoverflow.com/questions/3203286/how-to-create-a-read-only-class-property-in-python
-    class Meta(type(cls)):
+    class Meta(type(cls)):  # pyright: ignore[reportUntypedBaseClass]
         def __iter__(self):
             for f in self.features:
                 if f.is_scalar:
@@ -48,10 +51,14 @@ def classproperty_support(cls: Type[T]) -> Type[T]:
             if item == "__no_type_check__" or item == "shape":
                 raise AttributeError(f"Invalid attribute '{item}'.")
 
+            if (res := self.__chalk_notebook_feature_expressions__.get(item)) is not None:
+                return res
+
             builder: FeatureClassErrorBuilder = self.__chalk_error_builder__
             builder.invalid_attribute(
-                self.namespace,
-                item,
+                root_feature_str=self.namespace,
+                root_is_feature_class=True,
+                item=item,
                 candidates=[],
                 back=1,
             )
