@@ -4,7 +4,7 @@ import collections.abc
 import datetime
 import enum
 import functools
-from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Protocol, Sequence, TypeVar, Union
+from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Protocol, Sequence, TypeVar, Union, cast
 
 import pyarrow as pa
 from typing_extensions import Self, TypeGuard
@@ -44,6 +44,9 @@ class ParsedExpr(Protocol):
     def __add__(self, __value: Any, /) -> Self:
         ...
 
+    def __mul__(self, __value: Any, /) -> Self:
+        ...
+
     def __sub__(self, __value: Any, /) -> Self:
         ...
 
@@ -82,6 +85,10 @@ class FilterConverter(Protocol[TExpr]):
         """Expression for checking null"""
         ...
 
+    def call(self, function_name: str, *args: TExpr) -> TExpr:
+        """Call a system-defined function."""
+        ...
+
     def is_not_null(self, expr: TExpr) -> TExpr:
         """Should be equivalent to `not is_null(expr)`."""
         ...
@@ -220,8 +227,7 @@ class FilterConverter(Protocol[TExpr]):
         if f.operation in ("in", "not in"):
             assert lhs_converter is not None
             assert self._is_expr(lhs)
-            assert isinstance(rhs, collections.abc.Iterable)
-            ret = self._expr_is_in_sequence_lit(lhs, rhs, lhs_converter.pyarrow_dtype)
+            ret = self._expr_is_in_sequence_lit(lhs, cast(Any, rhs), lhs_converter.pyarrow_dtype)
             if f.operation == "not in":
                 ret = ~ret
         elif f.operation == "==" or f.operation == "!=":
@@ -323,7 +329,7 @@ class PolarsFilterConverter(FilterConverter["pl.Expr"]):
         return isinstance(o, pl.Expr)
 
     def cast(self, expr: pl.Expr, dtype: pa.DataType) -> pl.Expr:
-        raise NotImplemented
+        raise NotImplementedError
 
     def col(self, name: str) -> pl.Expr:
         import polars as pl
@@ -365,6 +371,7 @@ class PolarsFilterConverter(FilterConverter["pl.Expr"]):
             # If given a datetype with a timezone, and we have a target timezone on the datetime type,
             # we will first interpret the datetime in polars with the provided time zone, and then convert it to the desired time zone
             # Polars does not allow literal datetimes with a timezone to be directly interpreted as a datetime with time zone
+            assert pl_dtype.time_unit is not None, f"polars dtype {pl_dtype} incorrectly set with no time unit."
             return pl.lit(value, pl.Datetime(pl_dtype.time_unit)).dt.convert_time_zone(pl_dtype.time_zone)
 
         return pl.lit(value, dtype=pl_dtype)
@@ -383,6 +390,20 @@ class PolarsFilterConverter(FilterConverter["pl.Expr"]):
     def _get_struct_adapter(self, dtype: pa.StructType) -> StructAdapter[pl.Expr]:
         return _PolarsStructAdapter(dtype)
 
+    def call(self, function_name: str, *args: pl.Expr) -> pl.Expr:
+        import polars as pl
+
+        well_known_functions = {
+            "greatest": pl.max_horizontal,
+            "least": pl.min_horizontal,
+            "coalesce": pl.coalesce,
+        }
+        try:
+            fn = well_known_functions[function_name]
+        except KeyError:
+            raise ValueError(f"Unrecognized function {function_name}.")
+        return fn(*args)
+
 
 def convert_filters_to_pl_expr(
     filters: Sequence[Filter],
@@ -424,9 +445,9 @@ def filter_data_frame(
     underlying: Union[pl.DataFrame, pl.LazyFrame],
     namespace: Optional[str],
 ) -> Union[pl.DataFrame, pl.LazyFrame]:
+
     # Use the Chalk projection / selection syntax, where we support our Filter objects and
     # selection by column name
-    from chalk.features.feature_set import FeatureSetBase
 
     projections: list[str] = []
     filters: List[Filter] = []
@@ -446,7 +467,11 @@ def filter_data_frame(
         if key_error_or_none is not None:
             raise key_error_or_none
     # now = datetime.datetime.now(tz=datetime.timezone.utc)
-    timestamp_feature = None if namespace is None else FeatureSetBase.registry[namespace].__chalk_ts__
+    from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY
+
+    timestamp_feature = (
+        None if namespace is None else CURRENT_FEATURE_REGISTRY.get().get_feature_sets()[namespace].__chalk_ts__
+    )
     pl_expr = convert_filters_to_pl_expr(filters, underlying.schema, timestamp_feature, now)
     df = underlying
     if pl_expr is not None:
