@@ -22,6 +22,7 @@ from typing import (
     Optional,
     Sequence,
     Set,
+    Tuple,
     Type,
     TypeVar,
     Union,
@@ -43,7 +44,7 @@ from chalk.features.filter import Filter, TimeDelta, ClauseJoinWithAndException
 from chalk.features.tag import Tags
 from chalk.features.underscore import Underscore
 from chalk.serialization.parsed_annotation import ParsedAnnotation
-from chalk.utils.collections import ensure_tuple, get_unique_item, OrderedSet
+from chalk.utils.collections import ensure_tuple, get_unique_item, FrozenOrderedSet, OrderedSet
 from chalk.utils.duration import CHALK_MAX_TIMEDELTA, Duration, parse_chalk_duration
 from chalk.utils.import_utils import get_type_checking_imports
 from chalk.utils.json import JSON, pyarrow_json_type
@@ -143,6 +144,12 @@ class WindowConfigResolved:
       - group_by = (Transactions.user_id,)
       - aggregate_on = None
       - aggregation = "count"
+
+    User.transactions[_.user_id.approx_top_k(k=3)
+      - namespace = "transactions"
+      - group_by = (Transactions.user_id,)
+      - aggregate_on = Transaction.user_id
+      - aggregation = "approx_top_k"
     """
 
     namespace: str
@@ -167,6 +174,9 @@ class WindowConfigResolved:
     as could be the case with count, we will
     pick the primary key"""
 
+    aggregation_kwargs: FrozenOrderedSet[Tuple[str, Any]]
+    """Only some aggregations allow kwargs, namely 'approx_top_k'."""
+
     pyarrow_dtype: pa.DataType
 
     filters: list[Filter]
@@ -921,6 +931,14 @@ class Feature(Generic[_TPrim, _TRich]):
         it represents multiple windowed features."""
         return self.is_scalar and len(self.window_durations) > 0
 
+    @property
+    def is_group_by_windowed(self):
+        return self.group_by_windowed is not None
+
+    @property
+    def has_window_materialization(self):
+        return self.window_materialization is not None
+
     @property
     def is_windowed_pseudofeature(self):
         """Whether the feature is an underlying windowed pseudofeature,
