@@ -6,28 +6,35 @@ from typing_extensions import TYPE_CHECKING, ParamSpec
 from chalk._lsp.error_builder import get_resolver_error_builder
 from chalk.features.tag import Environments
 from chalk.state import KeyedState
-from chalk.streams._file_source import FileSource
 from chalk.streams._kafka_source import KafkaSource
 from chalk.streams._kinesis_source import KinesisSource
-from chalk.streams._windows import Windowed, get_duration_secs, get_name_with_duration, windowed
+from chalk.streams._pubsub_source import PubSubSource
+from chalk.streams._windows import (
+    MaterializationWindowConfig,
+    Windowed,
+    get_name_with_duration,
+    group_by_windowed,
+    windowed,
+)
 from chalk.streams.base import StreamSource
 from chalk.utils import MachineType
 
 if TYPE_CHECKING:
     from chalk.features.resolver import ResolverProtocol, StreamResolver
 
-__all__ = [
-    "FileSource",
+__all__ = (
     "KafkaSource",
     "KeyedState",
     "KinesisSource",
+    "MaterializationWindowConfig",
+    "PubSubSource",
     "StreamSource",
     "Windowed",
-    "get_duration_secs",
     "get_name_with_duration",
+    "group_by_windowed",
     "stream",
     "windowed",
-]
+)
 
 P = ParamSpec("P")
 T = TypeVar("T")
@@ -44,13 +51,14 @@ def stream(
     parse: Optional[Callable[[T], V]] = None,
     keys: Optional[Dict[str, Any]] = None,
     timestamp: Optional[str] = None,
+    updates_materialized_aggregations: bool = True,
 ) -> Union[Callable[[Callable[P, T]], "ResolverProtocol[P, T]"], "ResolverProtocol[P, T]"]:
     """Decorator to create a stream resolver.
 
     Parameters
     ----------
     source
-        The streaming source, e.g. `KafkaSource(...)` or `KinesisSource(...)`.
+        The streaming source, e.g. `KafkaSource(...)` or `KinesisSource(...)` or `PubSubSource(...)`.
     mode
         This parameter is defined when the streaming resolver returns a windowed feature.
         Tumbling windows are fixed-size, contiguous and non-overlapping time intervals. You can think of
@@ -62,7 +70,7 @@ def stream(
         When you request the value of a continuous window feature, Chalk looks
         at all the messages received in the window and computes the value on-demand.
 
-        See more at https://docs.chalk.ai/docs/aggregations#window-modes
+        See more at https://docs.chalk.ai/docs/windowed-streaming#window-modes
     parse
         A callable that will interpret an input prior to the invocation of the resolver.
         Parse functions can serve many functions, including pre-parsing bytes,
@@ -74,10 +82,10 @@ def stream(
         This parameter is required for continuous resolvers.
         Features that are included here do not have to be explicitly returned in the stream resolver:
         the feature will automatically be set to the key value used for aggregation.
-        See more at https://docs.chalk.ai/docs/aggregations#keys
     timestamp
         An optional string specifying an input attribute as the timestamp used for windowed aggregations.
-        See more at https://docs.chalk.ai/docs/aggregations#custom-event-timestamping
+    updates_materialized_aggregations
+        If set to `False`, the stream resolver will not update materialized aggregations, but is still eligible for ETL.
 
     Other Parameters
     ----------------
@@ -112,6 +120,7 @@ def stream(
     assert caller_frame is not None
     caller_globals = caller_frame.f_globals
     caller_locals = caller_frame.f_locals
+    del frame
     from chalk.features.resolver import parse_and_register_stream_resolver
 
     def decorator(fn: Callable[P, T]) -> "StreamResolver[P,T]":
@@ -133,6 +142,7 @@ def stream(
             parse=parse,
             keys=keys,
             timestamp=timestamp,
+            updates_materialized_aggregations=updates_materialized_aggregations,
         )
 
     return decorator
